# Bouncing LED par KenKen pour le Raspberry Pi Pico
# Télécharger ws2812b.py avec le lien ci-dessous et
# enregistrez le sur le Raspberry Pi Pico avec le même nom
# https://github.com/benevpi/pico_python_ws2812b
# Version française framboise314 - Mai 2021

from machine import Pin
import time
import ws2812b

LED_NUM = 288                               # Nombre de Neopixels sur le ruban
DIN_GPIO = 0                                # GPIO sur lequel le ruban est connecté
SENS_GPIO = Pin(15, Pin.IN, Pin.PULL_UP)    # GPIO pour inverser le sens de défilement avec R de pull-up
LUMIERE_GPIO = Pin(14, Pin.IN, Pin.PULL_UP) # GPIO pour modifier la luminosité 40% ou 80% avec R de pull-up
REVERSE = False                             # If True, LED block will fall off from the end of the LED strip

BLOCK = 8      # Longueur d'un bloc de LEDs - Défaut 8
BRIGHT = 40    # Luminosité des LED - Défaut 40 
GRAVITY = 20   # Accélération de la gravité - Défaut 20

# Coefficient de rebond = R1/R2
R1 = 3         # Coefficient de rebond R1 - Défaut 3
R2 = 5         # Coefficient de rebond R2 - Défaut 5
REFMINV = -80  # Vitesse minimum de rebond (Ajuster si le rebond ne s'arrête pas) - Défaut -80

# création de l'instance du ruban
strip = ws2812b.ws2812b(LED_NUM, 0, DIN_GPIO)

def set_block(pos, c):
# Crée un bloc de LEDs
# pos: Position de début du bloc de LEDs
# c: Couleur 0=noir, 1=rouge ... 8=vert ... 16=bleu ... 24=rouge

    if c == 0:
        r = 0; g = 0; b = 0
    elif c <= 8:
        r = (8 - c) * BRIGHT >> 3
        g = c * BRIGHT >>3
        b = 0
    elif c <= 16:
        r = 0
        g = (16 - c) * BRIGHT >> 3
        b = (c - 8) * BRIGHT >> 3
    elif c <= 24:
        r = (c - 16) * BRIGHT >> 3
        g = 0
        b = (24 - c) * BRIGHT >> 3
    else:
        return
    for i in range(BLOCK):
        if pos < 0:
            return
        if REVERSE:
            strip.set_pixel(LED_NUM - pos -1, r, g, b)
        else:
            strip.set_pixel(pos, r, g, b)
        pos -= 1

col = 1 # Couleur du blog de LED qui chute

# Tester si on doit inverser le sens de chute
if SENS_GPIO.value() == 0 :
    REVERSE = True

# Modifier la luminosité à 80% - Défaut => 40%
if LUMIERE_GPIO.value() == 0 :
    BRIGHT = 80
    
    
while True:
    # Note: La position et la vitesse sont décalés de 8 bits vers la gauche pour travailler avec des entiers
    bottom = LED_NUM << 8 # Configurer la fin sur la dernière LED du ruban

    # Boucler la chute des blocs
    while bottom >0:      # Répéter jusqu'à ce que la fin dépasse le haut du ruban
        y = 0             # Début du bloc de LEDs
        v = 0             # Vitesse de chute
        while y < bottom: # Répéter tant que le bloc de Leds est au dessus de la fin
            set_block(y >> 8, col) # Configurer les LEDs du bloc
            strip.show()           # Afficher le ruban   
            time.sleep_ms(5)       # Attendre 5 ms 
            set_block(y >> 8, 0)   # Effacer les LEDs
            v += GRAVITY           # Mettre à jour la vitesse de chute
            y += v                 # Mettre à jour la position
            if y >= bottom:        # Quand on atteint la fin
                y = bottom - 256   #  Mettre la position du bloc de LEDs à Une position au dessus de la fin
                v = -v * R1 // R2  #  Rebondir
                if v > REFMINV:    #  Arrêter quand la vitesse de rebond est inférieure à REFMINV
                    set_block((bottom >> 8) - 1, col) # Fixer le bloc de LEDs à une position au dessus de la fin
                    bottom -= BLOCK << 8         # Augmenter la position de fin de la longueur d'un bloc
                    col = (col + 18) % 24 + 1    # Passer à la couleur suivante

    strip.show()
    time.sleep_ms(500)

    y = 0 # Position du bout de la ligne de LEDs
    v = 0 # Vitesse de chute

    # On vide toute la ligne de LEDs
    while y < (LED_NUM << 8): # Répéter jusqu'à la disparition de l'extrémité de la ligne de LEDs
        y += v                # Mettre à jour la position
        v += GRAVITY // 2     # Mettre à jour la vitesse de chute (avec frottement)
        for i in range(LED_NUM-1, (y >> 8) - 1, -1):
            y1 = (i << 8) + (y & 0xff) - v # Position actuelle de la LED arrivant en position 1
            if y1 >= y:       # Copier la couleur de la LED en position y1 en i
                if REVERSE:
                    strip.pixels[LED_NUM - i -1] = strip.pixels[LED_NUM - (y1 >> 8) -1]
                else :
                    strip.pixels[i] = strip.pixels[y1 >> 8]
            else:             # Eteindre les LEDs au dessus du bout de la ligne de LEDs
                if REVERSE:
                    strip.set_pixel(LED_NUM - i -1, 0, 0, 0)
                else :
                    strip.set_pixel(i, 0, 0, 0)
        strip.show()
        time.sleep_ms(5)

    time.sleep_ms(500)        # Attendre 500 ms